/* This file was generated by syncqt. */
#ifndef QT_QTCORE5COMPAT_VERSION_H
#define QT_QTCORE5COMPAT_VERSION_H

#define QTCORE5COMPAT_VERSION_STR "6.2.4"

#define QTCORE5COMPAT_VERSION 0x060204

#endif // QT_QTCORE5COMPAT_VERSION_H
